# coding: utf8
import json
from django.conf import settings
from m3.actions.results import OperationResult
from sinops.settings import api_settings
from sinops.interface import ClientInterfaceBase

from .ui import SignAskWin


class ExtJSClientInterface(ClientInterfaceBase):

    u"""ExtJS интерфейс к пакету sinops."""

    SIGNATURE_REQUIRE_MSG = u'm3-sinops-signature-required'
    VERIFY_ERROR_MSG = u'Ошибка проверки подписи.'

    _propagate_operation_error = True

    def ask_signature_result(self, request, operation):
        u"""
        Возвращает ``OperationResult`` с запросом на подписание.

        :param operation: Выполняемая операция.
        :type operation: sinops.operation.SignedOperation
        """
        win = SignAskWin()
        win.display_content.value = operation.get_content_to_show()

        win.options = json.dumps({
            'content_to_sign': operation.get_content_to_sign(),
            'request_url': request.path,
            'request_params': dict(request.POST),
            'tsa_address': api_settings.TSA_SERVICE_ADDRESS,
        })
        return OperationResult(
            success=False,
            message=self.SIGNATURE_REQUIRE_MSG,
            code=win.get_script(),
        )

    def operation_completed_result(self, request, operation, value):
        return value

    def verify_error_result(self, request, operation, err):
        msg = unicode(err) or self.VERIFY_ERROR_MSG
        return OperationResult(success=False, message=msg)

    def operation_error_result(self, request, operation, err):
        if not self._propagate_operation_error:
            msg = u'Ошибка выполнения операции. %s' % unicode(err)
            return OperationResult(success=False, message=msg)
