
Ext.define('Ext.m3.SinopsSigner', {

    extend: 'Ext.util.Observable',

    PLUGIN_UNAVAILABLE_MSG: (
        'Выполняемое вами действие должно быть подписано с помощью ЭЦП, ' +
        'для создания которой необходим установленный "КриптоПро ЭЦП Browser plug-in". ' +
        'Пожалуйста проверьте настройки браузера!'
    ),

    constructor: function(config){
        this.callParent(arguments);

        this.options = Ext.apply({
            signature_param_name: '_signed_message',
            context_param_name: '_context_data'
        }, config.options);

        this.signWin = config.win;
        var winEl = this.signWin.getEl();

        this.pluginAwaitingMask = new Ext.LoadMask(winEl, {msg: 'Ожидание плагина'});
        this.signingProcessMask = new Ext.LoadMask(winEl, {msg: 'Формирование подписи'});
    },

    requirePlugin: function(onReady, onFail){
        var me = this;
        this.pluginAwaitingMask.show();

        function chek(){
            if(CPro.getPluginVersion() === undefined){
                 Ext.Msg.show({
                   title:'Не найден КриптоПро ЭЦП Browser plug-in!',
                   msg: me.PLUGIN_UNAVAILABLE_MSG,
                   buttons: {'yes': 'Повторить', 'cancel': 'Отмена'},
                   icon: Ext.MessageBox.ERROR,
                   fn: function (btn, text, opt) {
                        if (btn == 'yes') {
                            chek();
                        }else if(btn =='cancel'){
                            onFail();
                        }
                    }
                });
            }else{
                me.pluginAwaitingMask.hide();
                onReady();
            }
        }
        setTimeout(chek, 50);
    },

    getSignedMessage: function(){
        return this._signedMessage;
    },

    getOriginContent: function(){
        return this._originContent;
    },

    getSignOptions: function(){
        return this._signOptions;
    },

    getContext: function(){
        var signOpt = this.getSignOptions();
        var ctx = {
            signature_type: 1,
            signature_options: {
                detached: signOpt.detached,
                content_encoding: 'utf-16-le',
            }
        };
        switch (signOpt.signatureType){
            case CPro.const.CADESCOM_CADES_BES:
                ctx.signature_type = 1;
                break;
            case CPro.const.CADESCOM_CADES_X_LONG_TYPE_1:
                ctx.signature_type = 2;
                break;
        }
        return ctx;
    },

    getSignatureParams: function(){
        var params = {};
        params[this.options.signature_param_name] = this.getSignedMessage();
        params[this.options.context_param_name] = JSON.stringify(this.getContext());
        return params;
    },

    showSuccessMsg: function(msgBoxConfig){
        var conf = {
            title: 'Операция выполнена.',
            msg: 'Подпись была успешно проверена. Вы можете ' +
                  this.self.generateDownloadLink(this.getSignedMessage()) +
                 ' подписанный вами документ. ',
            buttons: Ext.MessageBox.OK,
            closable: false,
            icon: Ext.MessageBox.INFO
        };
        conf = Ext.apply(conf, msgBoxConfig);
        return Ext.Msg.show(conf);
    },

    sign: function(cert, content, signOptions){
        if(!cert || !content){
            throw new Error('Invalid certificate or content!');
        }
        var me = this;

        if (this.fireEvent('beforesign', cert, content, signOptions)) {

            me.signingProcessMask.show();

            //Так как формирование ЭЦП плагином КриптоПро происходит в
            //синхронном режиме, то изменения в DOM не будут отображены
            //браузером, пока текущий поток выполнения JS кода не будет
            //закончен.(В следствии этого, например, маска формирования
            //подписи signingProcessMask не будет отображена)
            //Для обхода этой проблемы вызываем функцию формирования подписи
            //асинхронно с некоторой задержкой.
            setTimeout(function(){
                var _error = true;
                try{
                    me._signedMessage = CPro.createSignature(cert, content, signOptions);
                    me._originContent = content;
                    me._signOptions = signOptions;
                    _error = false;
                }catch(err){
                    if (me.fireEvent('signfailed', err, cert, content, signOptions)){
                        if(err instanceof CPro.SignCreateError){
                            Ext.Msg.show({
                                title:'Ошибка создания подписи!',
                                msg: 'Подробности ошибки: ' + (err.message || err.name),
                                buttons: Ext.Msg.OK,
                                icon: Ext.MessageBox.ERROR
                            });
                            return;
                        }else{
                            throw err;
                        }
                    }
                }finally{
                    me.signingProcessMask.hide();
                }
                if(!_error){
                    me.fireEvent('aftersign', me._signedMessage, content);
                }
            }, 50);
        }
    },

    statics: {
        generateDownloadLink: function(base64data, filename, text){
            filename = filename || 'signed_message.der';
            text = text || 'скачать документ';
            return (
                '<a download="'+ filename + '"' +
                'href="data:application/octet-stream;charset=utf-8;base64,' +
                base64data + '" >' + text + '</a>'
            );
        }
    }
});
